/************************************************************************************
hazardmeshirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//***************************************************************************************//
//* HazardMesh.cpp							     2010.02.06 *//
//*											*//
//* Hazard Mesh RTMプログラム [Ver.1.0.0]						*//
//*											*//
//* コンフィギュレーション								*//
//*	01_gui			: GUI動作						*//
//*				  [off, on]						*//
//*	02_guiaddress		: GUIプログラムアドレス					*//
//*	03_ip			: IPアドレス						*//
//*	04_sendport		: 送信ポート番号					*//
//*	05_receiveport		: 受信ポート番号					*//
//***************************************************************************************//


// -*- C++ -*-
/*!
 * @file  HazardMesh.cpp
 * @brief Mesh Map of Hazard
 * $Date$
 *
 * $Id$
 */

#include "HazardMesh.h"

//-- [変数宣言] ---------------------------------------------------------------------------------//
int wait_flag = 1;											// フラグ格納変数 [読み込み用]
//-----------------------------------------------------------------------------------------------//

// Module specification
// <rtc-template block="module_spec">
static const char* hazardmesh_spec[] =
  {
    "implementation_id", "HazardMesh",
    "type_name",         "HazardMesh",
    "description",       "Mesh Map of Hazard",
    "version",           "1.0.0",
    "vendor",            "IRS",
    "category",          "Map",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.01_gui", "on",
    "conf.default.02_guiaddress", "../gui/hazardmesh",
    "conf.default.03_ip", "127.0.0.1",
    "conf.default.04_sendport", "10006",
    "conf.default.05_receiveport", "10005",

    ""
  };
// </rtc-template>

HazardMesh::HazardMesh(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_MapDataIn("MapData", m_MapData),
    m_MeshOrderIn("MeshOrder", m_MeshOrder),
    m_MapOrderOut("MapOrder", m_MapOrder),
    m_MeshDataOut("MeshData", m_MeshData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("MapData", m_MapDataIn);
  registerInPort("MeshOrder", m_MeshOrderIn);
  
  // Set OutPort buffer
  registerOutPort("MapOrder", m_MapOrderOut);
  registerOutPort("MeshData", m_MeshDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

HazardMesh::~HazardMesh()
{
}


RTC::ReturnCode_t HazardMesh::onInitialize()
{
	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("01_gui", m_01_gui, "on");
	bindParameter("02_guiaddress", m_02_guiaddress, "../gui/hazardmesh");
	bindParameter("03_ip", m_03_ip, "127.0.0.1");
	bindParameter("04_sendport", m_04_sendport, "10006");
	bindParameter("05_receiveport", m_05_receiveport, "10005");

	// </rtc-template>


	//-- [初期化] ---------------------------------------------------------------------------//

	m_MapOrder.location.length(DATASIZE);								// メモリ確保 [location]
	m_MapOrder.mapset.length(DATASIZE);								// メモリ確保 [mapset]
	m_MapOrder.type.length(DATASIZE);								// メモリ確保 [type]
	m_MeshData.location.length(DATASIZE);								// メモリ確保 [location]
	m_MeshData.mapset.length(DATASIZE);								// メモリ確保 [mapset]
	m_MeshData.type.length(DATASIZE);								// メモリ確保 [type]
	m_MeshData.explanation.length(DATASIZE);							// メモリ確保 [explanation]
	m_MeshData.category.length(18);									// メモリ確保 [category]

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t HazardMesh::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HazardMesh::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HazardMesh::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t HazardMesh::onActivated(RTC::UniqueId ec_id)
{


	//-- [初期化] ---------------------------------------------------------------------------//

	gui_flag     = 0;										// GUI動作フラグ初期化

	sec          = 0;										// タイムスタンプ初期化 [sec]
	nsec         = 0;										// タイムスタンプ初期化 [nsec]
	trigger      = 0;										// 動作トリガー初期化
	mode         = 0;										// アクセスモード初期化
	strcpy(location, "");										// 地域名初期化
	strcpy(mapset, "");										// 利用者マップセット初期化
	strcpy(type, "");										// データタイプ初期化
	origin_x     = 0;										// 対象原点x座標初期化
	origin_y     = 0;										// 対象原点y座標初期化
	size_x       = 0;										// 対象範囲x座標初期化
	size_y       = 0;										// 対象範囲y座標初期化
	pixel_size_x = 0;										// ピクセルサイズx座標初期化
	pixel_size_y = 0;										// ピクセルサイズy座標初期化
	strcpy(send_data, "");										// 送信データ初期化
	trigger_flag = 0;										// 動作トリガーフラグ初期化

	sec_2          = 0;										// タイムスタンプ初期化 [sec]
	nsec_2         = 0;										// タイムスタンプ初期化 [nsec]
	mode_2         = 0;										// アクセスモード初期化
	strcpy(location_2, "");										// 地域名初期化
	strcpy(mapset_2, "");										// 利用者マップセット初期化
	strcpy(type_2, "");										// データタイプ初期化
	origin_x_2     = 0;										// 対象原点x座標初期化
	origin_y_2     = 0;										// 対象原点y座標初期化
	size_x_2       = 0;										// 対象範囲x座標初期化
	size_y_2       = 0;										// 対象範囲y座標初期化
	strcpy(explanation_2, "");									// 地図説明初期化
	map_origin_x_2 = 0;										// 地図原点x座標初期化
	map_origin_y_2 = 0;										// 地図原点y座標初期化
	map_size_x_2   = 0;										// 地図サイズx座標初期化
	map_size_y_2   = 0;										// 地図サイズy座標初期化
	pixel_size_x   = 0;										// ピクセルサイズx座標初期化
	pixel_size_y   = 0;										// ピクセルサイズy座標初期化

	x = 0;												// x座標個数初期化
	y = 0;												// y座標個数初期化

	//---------------------------------------------------------------------------------------//


	//-- [出力ポート初期化] -----------------------------------------------------------------//

	// タイムスタンプ初期化
	m_MapOrder.tm.sec  = 0;										// 初期化 [m]
	m_MapOrder.tm.nsec = 0;										// 初期化 [m]

	// 動作トリガー初期化
	m_MapOrder.trigger = 0;										// 初期化 [trigger]

	// モード初期化
	m_MapOrder.mode = 0;										// 初期化 [mode]

	// 地域名初期化
	m_MapOrder.location[0] = 0x00;									// 初期化 [location]

	// 利用者マップセット名初期化
	m_MapOrder.mapset[0] = 0x00;									// 初期化 [mapset]

	// データタイプ初期化
	m_MapOrder.type[0] = 0x00;									// 初期化 [type]

	// 対象原点x座標初期化
	m_MapOrder.origin_x = 0;									// 初期化 [origin_x]

	// 対象原点y座標初期化
	m_MapOrder.origin_y = 0;									// 初期化 [origin_y]

	// 対象範囲x座標初期化
	m_MapOrder.size_x = 0;										// 初期化 [size_x]

	// 対象範囲y座標初期化
	m_MapOrder.size_y = 0;										// 初期化 [size_y]

	//---------------------------------------------------------------------------------------//


	//-- [コンフィギュレーション読み込み] ---------------------------------------------------//

	strcpy(gui,       m_01_gui.c_str());								// データ格納 [GUI動作]
	strcpy(gui_addr,  m_02_guiaddress.c_str());							// データ格納 [GUIプログラムアドレス]
	strcpy(ip,        m_03_ip.c_str());								// データ格納 [IPアドレス]
	strcpy(send_port, m_04_sendport.c_str());							// データ格納 [送信ポート番号]
	strcpy(recv_port, m_05_receiveport.c_str());							// データ格納 [受信ポート番号]

	//---------------------------------------------------------------------------------------//


	//-- [デバッグconfiguration] ------------------------------------------------------------//

	#ifndef DEBUG_CONFIGURATION									// デバッグ表示開始

	printf("\n[configuration]\n");									// label
	printf("01_gui         = %s\n", gui);								// gui
	printf("02_guiaddress  = %s\n", gui_addr);							// guiaddress
	printf("03_ip          = %s\n", ip);								// ip
	printf("04_sendport    = %s\n", send_port);							// sendport
	printf("05_receiveport = %s\n\n", recv_port);							// receiveport
	fflush(stdout);											// バッファフラッシュ [printf表示用]

	#endif												// デバッグ表示終了

	//---------------------------------------------------------------------------------------//


	//-- [コンフィギュレーションチェック] ---------------------------------------------------//

	// GUI動作チェック
	if(strcmp("off", gui) == 0) gui_flag = 0;							// GUI機能OFF
	else if(strcmp("on", gui) == 0) gui_flag = 1;							// GUI機能ON
	else{

		printf("configuration error:01_gui = %s\n", gui);					// エラーメッセージ
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		return RTC::RTC_ERROR;									// 戻り値 [異常終了]

	}

	//---------------------------------------------------------------------------------------//


	//-- [GUI関連] --------------------------------------------------------------------------//

	if(gui_flag == 1){										// GUI機能ON


		//-- [GUIプログラム起動] --------------------------------------------------------//

		strcpy(command, gui_addr);								// 実行プログラム
		strcat(command, " ");									// 空白
		strcat(command, ip);									// オプション [IPアドレス]
		strcat(command, " ");									// 空白
		strcat(command, recv_port);								// オプション [受信ポート番号]
		strcat(command, " ");									// 空白
		strcat(command, send_port);								// オプション [送信ポート番号]
		strcat(command, " &");									// オプション [バックグラウンド]
		system(command);									// GUIプログラム起動
		usleep(WAITTIME);									// 待機 WAITTIME [us]

		//-------------------------------------------------------------------------------//


		//-- [UDP通信関連初期化] --------------------------------------------------------//

		memset(send_data, 0, sizeof(send_data));						// 送信データ初期化
		memset(recv_data, 0, sizeof(recv_data));						// 受信データ初期化

		//-------------------------------------------------------------------------------//


		//-- [ソケット作成] -------------------------------------------------------------//
		//-										-//
		//- プロトコル・ファミリ							-//
		//- AF_UNIX, AF_LOCAL	: ローカル通信						-//
		//- AF_INET		: IPv4 インターネットプロトコル				-//
		//- AF_INET6		: IPv6 インターネットプロトコル				-//
		//- AF_IPX		: IPX-NoveIIプロトコル					-//
		//- AF_NETLINK		: カーネルユーザデバイス				-//
		//- AF_X25		: ITU-T X.25/ISO-8208プロトコル				-//
		//- AF_AX25		: アマチュア無線AX.25プロトコル				-//
		//- AF_ATMPVC		: ATMPVC						-//
		//- AF_APPLETALK	: アップルトーク					-//
		//- AF_PACKET		: 低レベルパケットインターフェイス			-//
		//- 										-//
		//- 通信方式									-//
		//- SOCK_STREAM		: バイトストリーム	[信頼性有:順序保証有:双方向]	-//
		//- SOCK_DGRAM		: データグラム		[信頼性無:固定最大長]		-//
		//- SOCK_SEQPACKET	: データグラム		[信頼性有:固定最大長:双方向]	-//
		//- SOCK_RAW		: 生ネットワーク					-//
		//- SOCK_RDW		: データグラム		[信頼性有:順序保証無]		-//
		//- SOCK_PACKET		: 廃止							-//
		//-------------------------------------------------------------------------------//

		if((send_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){					// ソケット作成 [送信データ用]

			printf("socket error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		if((recv_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){					// ソケット作成 [受信データ用]

			printf("socket error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//-------------------------------------------------------------------------------//


		//-- [アドレス設定] -------------------------------------------------------------//

		send_addr.sin_family      = AF_INET;							// アドレスファミリ
		send_addr.sin_port        = htons(atoi(send_port));					// ポート番号 [バイトオーダ]
		send_addr.sin_addr.s_addr = inet_addr(ip);						// IPアドレス [バイトオーダ]

		recv_addr.sin_family      = AF_INET;							// アドレスファミリ
		recv_addr.sin_port        = htons(atoi(recv_port));					// ポート番号 [バイトオーダ]
		recv_addr.sin_addr.s_addr = inet_addr(ip);						// IPアドレス [バイトオーダ]

		//-------------------------------------------------------------------------------//


		//-- [受信設定] -----------------------------------------------------------------//

		bind(recv_sock, (struct sockaddr *)&recv_addr, sizeof(recv_addr));			// バインド [受信データ]
		addr_size = sizeof(recv_addr);								// アドレス格納構造体サイズ格納

		//-------------------------------------------------------------------------------//


		//-- [シグナルハンドル設定] -----------------------------------------------------//

		saio.sa_handler = signal_handler_IO;							// シグナルハンドル
		sigemptyset(&saio.sa_mask);								// 追加マスクシグナル
		saio.sa_flags = 0;									// 動作詳細設定フラグ
		saio.sa_restorer = NULL;								// オボソリート [= NULL]

		//-------------------------------------------------------------------------------//


		//-- [シグナルハンドル調査/変更] ------------------------------------------------//

		sigaction(SIGIO, &saio, NULL);								// シグナルアクション調査/変更

		//-------------------------------------------------------------------------------//


		//-- [非同期通信モード設定] -----------------------------------------------------//

		fcntl(recv_sock, F_SETOWN, getpid());							// シグナル受け取り設定 [プロセス]
		fcntl(recv_sock, F_SETFL, O_NONBLOCK | FASYNC);						// 非同期通信モード設定

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t HazardMesh::onDeactivated(RTC::UniqueId ec_id)
{


	//-- [GUI関連] --------------------------------------------------------------------------//

	if(gui_flag == 1){										// GUI機能ON


		//-- [GUIプログラム終了コマンド送信] --------------------------------------------//

		strcpy(send_data, "quit");								// 送信データ設定

		if(sendto(send_sock, send_data, strlen(send_data), 0,
			(struct sockaddr *)&send_addr, sizeof(send_addr)) < 0){				// データ送信

			printf("send error!!\n");							// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		//-------------------------------------------------------------------------------//


		//-- [ソケットディスクリプタクローズ] -------------------------------------------//

		close(send_sock);									// ソケットディスクリプタクローズ
		close(recv_sock);									// ソケットディスクリプタクローズ

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

RTC::ReturnCode_t HazardMesh::onExecute(RTC::UniqueId ec_id)
{


	//-- [データ更新] -----------------------------------------------------------------------//

	if(m_MeshOrderIn.isNew()){									// 更新チェック

		m_MeshOrderIn.read();									// データ読み込み


		//-- [データ格納] ---------------------------------------------------------------//

		sec     = m_MeshOrder.tm.sec;								// タイムスタンプ格納 [sec]
		nsec    = m_MeshOrder.tm.nsec;								// タイムスタンプ格納 [nsec]
		trigger = m_MeshOrder.trigger;								// 動作トリガー格納

		if(trigger == 0) trigger_flag = 0;							// 動作トリガーフラグクリア
		else if((trigger == 1) && (trigger_flag == 0)) trigger_flag = 1;			// 動作トリガーフラグセット
		else trigger_flag = -1;									// 動作トリガーフラグセット

		mode         = m_MeshOrder.mode;							// アクセスモード格納

		for(i = 0; i < BUFSIZE; i++){

			location[i] = m_MeshOrder.location[i];						// 地域名格納
			if(m_MeshOrder.location[i] == 0x00) break;					// 終端文字チェック

		}

		for(i = 0; i < BUFSIZE; i++){

			mapset[i] = m_MeshOrder.mapset[i];						// 利用者マップセット格納
			if(m_MeshOrder.mapset[i] == 0x00) break;					// 終端文字チェック

		}

		for(i = 0; i < BUFSIZE; i++){

			type[i] = m_MeshOrder.type[i];							// データタイプ格納
			if(m_MeshOrder.type[i] == 0x00) break;						// 終端文字チェック

		}

		origin_x     = m_MeshOrder.origin_x;							// 対象原点x座標格納
		origin_y     = m_MeshOrder.origin_y;							// 対象原点y座標格納
		size_x       = m_MeshOrder.size_x;							// 対象範囲x座標格納
		size_y       = m_MeshOrder.size_y;							// 対象範囲y座標格納
		pixel_size_x = m_MeshOrder.pixel_size_x;						// ピクセルサイズx座標格納
		pixel_size_y = m_MeshOrder.pixel_size_y;						// ピクセルサイズy座標格納

		//-------------------------------------------------------------------------------//


		//-- [デバッグinport] -----------------------------------------------------------//

		#ifndef DEBUG_INPORT									// デバッグ表示開始

		printf("\n[inport]\n");									// label
		printf("timestamp    = %ld.%ld\n", sec, nsec);						// timestamp
		printf("trigger      = %d\n", trigger);							// trigger
		printf("mode         = %d\n", mode);							// mode
		printf("location     = %s\n", location);						// location
		printf("mapset       = %s\n", mapset);							// mapset
		printf("type         = %s\n", type);							// type
		printf("origin_x     = %ld\n", origin_x);						// origin_x
		printf("origin_y     = %ld\n", origin_y);						// origin_y
		printf("size_x       = %ld\n", size_x);							// size_x
		printf("size_y       = %ld\n", size_y);							// size_y
		printf("pixel_size_x = %ld\n", pixel_size_x);						// pixel_size_x
		printf("pixel_size_y = %ld\n\n", pixel_size_y);						// pixel_size_y
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-- [トリガー] -----------------------------------------------------------------//

		if((trigger_flag == 0) || (trigger_flag == 1)){						// 動作トリガーチェック

			m_MapOrder.tm.sec   = sec;							// タイムスタンプ格納 [sec]
			m_MapOrder.tm.nsec  = nsec;							// タイムスタンプ格納 [nsec]
			m_MapOrder.trigger  = trigger;							// 動作トリガー格納
			m_MapOrder.mode     = mode;							// アクセスモード格納

			for(i = 0; i < DATASIZE; i++){

				m_MapOrder.location[i] = location[i];					// 地域名格納
				if(location[i] == 0x00) break;						// 終端文字チェック

			}

			for(i = 0; i < DATASIZE; i++){

				m_MapOrder.mapset[i] = mapset[i];					// 利用者マップセット格納
				if(mapset[i] == 0x00) break;						// 終端文字チェック

			}

			for(i = 0; i < DATASIZE; i++){

				m_MapOrder.type[i] = type[i];						// データタイプ格納
				if(type[i] == 0x00) break;						// 終端文字チェック

			}

			m_MapOrder.origin_x = origin_x;							// 対象原点x座標格納
			m_MapOrder.origin_y = origin_y;							// 対象原点y座標格納
			m_MapOrder.size_x   = size_x;							// 対象範囲x座標格納
			m_MapOrder.size_y   = size_y;							// 対象範囲y座標格納

			if(trigger_flag == 1) trigger_flag = -1;					// 動作トリガーセット

			m_MapOrderOut.write();								// データ出力

		}

		//-------------------------------------------------------------------------------//


	}else if(m_MapDataIn.isNew()){									// 更新チェック

		m_MapDataIn.read();									// データ読み込み


		//-- [データ格納] ---------------------------------------------------------------//

		sec_2          = m_MapData.tm.sec;							// タイムスタンプ格納 [sec]
		nsec_2         = m_MapData.tm.nsec;							// タイムスタンプ格納 [nsec]
		mode_2         = m_MapData.mode;							// アクセスモード格納

		for(i = 0; i < BUFSIZE; i++){

			location_2[i] = m_MapData.location[i];						// 地域名格納
			if(m_MapData.location[i] == 0x00) break;					// 終端文字チェック

		}

		for(i = 0; i < BUFSIZE; i++){

			mapset_2[i] = m_MapData.mapset[i];						// 利用者マップセット格納
			if(m_MapData.mapset[i] == 0x00) break;						// 終端文字チェック

		}

		for(i = 0; i < BUFSIZE; i++){

			type_2[i] = m_MapData.type[i];							// データタイプ格納
			if(m_MapData.type[i] == 0x00) break;						// 終端文字チェック

		}

		origin_x_2     = m_MapData.origin_x;							// 対象原点x座標格納
		origin_y_2     = m_MapData.origin_y;							// 対象原点y座標格納
		size_x_2       = m_MapData.size_x;							// 対象範囲x座標格納
		size_y_2       = m_MapData.size_y;							// 対象範囲y座標格納

		for(i = 0; i < BUFSIZE; i++){

			explanation_2[i] = m_MapData.explanation[i];					// 地図説明格納
			if(m_MapData.explanation[i] == 0x00) break;					// 終端文字チェック

		}

		map_origin_x_2 = m_MapData.map_origin_x;						// 地図原点x座標格納
		map_origin_y_2 = m_MapData.map_origin_y;						// 地図原点y座標格納
		map_size_x_2   = m_MapData.map_size_x;							// 地図サイズx座標格納
		map_size_y_2   = m_MapData.map_size_y;							// 地図サイズy座標格納
		pixel_size_x_2 = m_MapData.pixel_size_x;						// ピクセルサイズx座標格納
		pixel_size_y_2 = m_MapData.pixel_size_y;						// ピクセルサイズy座標格納


		//-- [メモリ確保] ---------------------------------------------------------------//

		pcategory = (char*)calloc(sizeof(m_MeshData.category), sizeof(unsigned char));		// メモリ確保

		for(i = 0; i < BUFSIZE; i++){

			*(pcategory + i) = m_MapData.category[i];					// カテゴリデータ格納
			if(m_MapData.category[i] == 0x00) break;					// 終端文字チェック

		}

		x = size_x_2 / pixel_size_x_2;								// x座標個数格納
		y = size_y_2 / pixel_size_y_2;								// y座標個数格納

		pmap = (unsigned long*)calloc(x * y, sizeof(unsigned long));				// メモリ確保

		if(pmap == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		for(i = 0; i < (unsigned long)(x * y); i++) *(pmap + i) = m_MapData.mapdata[i];		// 地図データ格納

		//-------------------------------------------------------------------------------//


		//-- [デバッグmapdata] ----------------------------------------------------------//

		#ifndef DEBUG_MAPDATA									// デバッグ表示開始

		printf("\n[mapdata]\n");								// label
		printf("timestamp    = %ld.%ld\n", sec_2, nsec_2);					// timestamp_2
		printf("mode         = %d\n", mode_2);							// mode_2
		printf("location     = %s\n", location_2);						// location_2
		printf("mapset       = %s\n", mapset_2);						// mapset_2
		printf("type         = %s\n", type_2);							// type_2
		printf("origin_x     = %ld\n", origin_x_2);						// origin_x_2
		printf("origin_y     = %ld\n", origin_y_2);						// origin_y_2
		printf("size_x       = %ld\n", size_x_2);						// size_x_2
		printf("size_y       = %ld\n", size_y_2);						// size_y_2
		printf("explanation  = %s\n", explanation_2);						// explanation_2
		printf("map_origin_x = %ld\n", map_origin_x_2);						// map_origin_x_2
		printf("map_origin_y = %ld\n", map_origin_y_2);						// map_origin_y_2
		printf("map_size_x   = %ld\n", map_size_x_2);						// map_size_x_2
		printf("map_size_y   = %ld\n", map_size_y_2);						// map_size_y_2
		printf("pixel_size_x = %ld\n", pixel_size_x_2);						// pixel_size_x_2
		printf("pixel_size_y = %ld\n", pixel_size_y_2);						// pixel_size_y_2
		printf("category     = %s\n", pcategory);						// category_2
		printf("mapdata      =\n");								// mapdata

		for(i = 0; i < y; i++){

			for(j = 0; j < x; j++) printf("%ld", *(pmap + x * i + j));			// データ表示
			printf("\n");									// 改行表示

		}

		printf("\n");										// 改行表示

		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		if(pixel_size_x < pixel_size_x_2) pixel_size_x = pixel_size_x_2;			// ピクセルサイズ処理
		if(pixel_size_y < pixel_size_y_2) pixel_size_y = pixel_size_y_2;			// ピクセルサイズ処理

		//-------------------------------------------------------------------------------//


		//-- [カテゴリ処理] -------------------------------------------------------------//

		j = 0;											// カウント初期化
		category_num = 0;									// カテゴリ番号初期化
		category_flag = 0;									// フラグクリア

		for(i = 0; i < BUFSIZE; i++){

			if((*(pcategory + i) == ',') || (*(pcategory + i) == 0x00)){			// 区切りチェック

				category_buf[category_num][j] = 0x00;					// 終端文字格納
				j = 0;									// カウント初期化

				if(category_num == 1){

					if(strcmp("hazard", category_buf[1]) == 0){			// カテゴリチェック

						hazard = atol(category_buf[0]);				// ハザード番号格納
						if(category_flag == 2) break;				// カテゴリ処理終了
						category_flag = 1;					// フラグチェック

					}else if(strcmp("safety", category_buf[1]) == 0){		// カテゴリチェック

						safety = atol(category_buf[0]);				// セーフティ番号格納
						if(category_flag == 1) break;				// カテゴリ処理終了
						category_flag = 2;					// フラグチェック

					}

					category_num = 0;						// カウント変更

				}else{

					category_num = 1;						// カウント変更

				}

				if(*(pcategory + i) == 0x00){						// 終端文字チェック

					printf("category error!! [hazard]\n");				// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

			}else{										// データチェック

				category_buf[category_num][j] = *(pcategory + i);			// データ格納
				j++;									// カウント

			}

		}

		//-------------------------------------------------------------------------------//


		//-- [ハザードメッシュ処理] -----------------------------------------------------//


		//-- [メモリ確保] ---------------------------------------------------------------//

		hazard_x = (unsigned long int)((size_x_2 + pixel_size_x - 1) / (pixel_size_x));		// ハザードマップx座標個数格納
		hazard_y = (unsigned long int)((size_y_2 + pixel_size_y - 1) / (pixel_size_y));		// ハザードマップy座標個数格納

		phazard = (unsigned long*)calloc(hazard_x * hazard_y + 1, sizeof(unsigned long));	// メモリ確保

		if(phazard == NULL){

			printf("memory securing error!!\n");						// メッセージ表示
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			return RTC::RTC_ERROR;								// 戻り値 [異常終了]

		}

		for(i = 0; i < hazard_y - 1; i++){

			for(j = 0; j < hazard_x; j++) *(phazard + hazard_x * i + j) = SAFETYNO;		// データ初期化

		}
		*(phazard + hazard_x * hazard_y) = 0x00;						// 終端文字格納

		//-------------------------------------------------------------------------------//


		//-- [ハザードデータ格納] -------------------------------------------------------//


		for(i = 0; i < y; i++){

			for(j = 0; j < x; j++){

				if(*(pmap + x * i + j) == hazard){					// ハザードチェック

					// x座標:(unsigned long int)(((j + 1) * pixel_size_x_2 - 1) / pixel_size_x)
					// y座標:(unsigned long int)(((i + 1) * pixel_size_y_2 - 1) / pixelsize_y)

					*(phazard + hazard_x * (unsigned long int)(((i + 1) * pixel_size_y_2 - 1) / pixel_size_y)
						+ (unsigned long int)(((j + 1) * pixel_size_x_2 - 1) / pixel_size_x))
						= HAZARDNO;						// データ格納

				}

			}

		}


		//-- [メモリ解放] ---------------------------------------------------------------//

		free(pmap);										// メモリ解放

		//-------------------------------------------------------------------------------//


		//-- [デバッグmeshdata] ---------------------------------------------------------//

		#ifndef DEBUG_MESHDATA									// デバッグ表示開始

		for(i = 0; i < hazard_y; i++){

			for(j = 0; j < hazard_x; j++)
				printf("%ld", *(phazard + hazard_x * i + j));				// データ表示

			printf("\n");									// 改行表示

		}

		printf("\n");										// 改行表示

		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//-------------------------------------------------------------------------------//


		//-------------------------------------------------------------------------------//


		//-- [GUI関連] ------------------------------------------------------------------//

		if(gui_flag == 1){									// GUI機能ON


			//-- [データ送信] -------------------------------------------------------//

			sprintf(send_data, "%ld,%ld,%d,%s,%s,%s,%ld,%ld,%ld,%ld,%s,%ld,%ld,%ld,%ld,%ld,%ld,%ld",
				sec_2, nsec_2, mode_2, location_2, mapset_2, type_2, origin_x_2, origin_y_2,
				hazard_x * pixel_size_x, hazard_y * pixel_size_y, explanation_2, map_origin_x_2,
				map_origin_y_2, map_size_x_2, map_size_y_2, pixel_size_x, pixel_size_y,
				hazard_x * hazard_y + 1);						// データ格納

			if(sendto(send_sock, send_data, strlen(send_data), 0,
				(struct sockaddr *)&send_addr, sizeof(send_addr)) < 0){			// データ送信

				printf("send error!!\n");						// メッセージ表示
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				return RTC::RTC_ERROR;							// 戻り値 [異常終了]

			}

			psend = (char*)calloc(hazard_x * hazard_y + 1, sizeof(char));			// メモリ確保
			for(i = 0; i < hazard_x * hazard_y; i++) *(psend + i) = *(phazard + i) + 0x30;	// データ格納
			*(psend + i) = 0x00;								// 終端文字格納

			for(i = 0; i < hazard_y; i++){

				if(sendto(send_sock, psend + hazard_x * i, hazard_x, 0,
					(struct sockaddr *)&send_addr, sizeof(send_addr)) < 0){		// データ送信

					printf("send error!!\n");					// メッセージ表示
					fflush(stdout);							// バッファフラッシュ [printf表示用]

					return RTC::RTC_ERROR;						// 戻り値 [異常終了]

				}

				usleep(1000);								// 待機

			}

			free(psend);									// メモリ解放

			//-----------------------------------------------------------------------//


		}

		//-------------------------------------------------------------------------------//


		//-- [出力データ格納] -----------------------------------------------------------//

		m_MeshData.tm.sec  = sec_2;								// タイムスタンプ格納 [sec]
		m_MeshData.tm.nsec = nsec_2;								// タイムスタンプ格納 [nsec]
		m_MeshData.mode    = mode_2;								// モード格納
		for(i = 0; i <= strlen(location_2); i++)
			m_MeshData.location[i] = location_2[i];						// 地域名格納
		for(i = 0; i <= strlen(mapset_2); i++)
			m_MeshData.mapset[i] = mapset_2[i];						// 利用者マップセット格納
		for(i = 0; i <= strlen(type_2); i++)
			m_MeshData.type[i] = type_2[i];							// データタイプ格納
		m_MeshData.origin_x = origin_x_2;							// 対象原点x座標格納
		m_MeshData.origin_y = origin_y_2;							// 対象原点y座標格納
		m_MeshData.size_x   = hazard_x * pixel_size_x;						// 対象範囲x座標格納
		m_MeshData.size_y   = hazard_y * pixel_size_y;						// 対象範囲y座標格納
		for(i = 0; i <= strlen(explanation_2); i++)
			m_MeshData.explanation[i] = explanation_2[i];					// 地図説明格納
		m_MeshData.map_origin_x = map_origin_x_2;						// 地図原点x座標格納
		m_MeshData.map_origin_y = map_origin_y_2;						// 地図原点y座標格納
		m_MeshData.map_size_x   = map_size_x_2;							// 地図サイズx座標格納
		m_MeshData.map_size_y   = map_size_y_2;							// 地図サイズy座標格納
		m_MeshData.pixel_size_x = pixel_size_x;							// ピクセルサイズx座標格納
		m_MeshData.pixel_size_y = pixel_size_y;							// ピクセルサイズy座標格納
		strcpy(pcategory, HAZARDCATEGORY);							// 一時カテゴリ格納
		for(i = 0; i < CATEGORYSIZE; i++) m_MeshData.category[i] = *(pcategory + i);		// カテゴリ格納
		m_MeshData.mapdata.length(hazard_x * hazard_y + 1);					// メモリ確保 [mapdata]
		for(i = 0; i < hazard_x * hazard_y; i++) m_MeshData.mapdata[i] = *(phazard + i);	// 地図情報格納
		m_MeshData.mapdata[i] = 0x00;								// 終端文字処理


		//-- [データ出力] ---------------------------------------------------------------//

		m_MeshDataOut.write();									// データ出力

		//-------------------------------------------------------------------------------//


		//-- [メモリ解放] ---------------------------------------------------------------//

		free(pcategory);									// メモリ解放
		free(phazard);										// メモリ解放
		m_MeshData.mapdata.length(0);								// メモリ解放 [mapdata]

		//-------------------------------------------------------------------------------//


		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	//-- [GUI関連] --------------------------------------------------------------------------//

	if(gui_flag == 1){										// GUI機能ON


		//-- [データ受信] ---------------------------------------------------------------//

		if(wait_flag == 0){

			// データ受信
			recv_len = recvfrom(recv_sock, recv_data, sizeof(recv_data), 0,
				(struct sockaddr *)&recv_addr, &addr_size);				// データ受信

			if(recv_len < 0){								// 受信チェック

				printf("receive error!!\n");						// メッセージ表示
				fflush(stdout);								// バッファフラッシュ [printf表示用]

				return RTC::RTC_ERROR;							// 戻り値 [異常終了]

			}


			//-- [デバッグrecv] -----------------------------------------------------//

			#ifndef DEBUG_RECV								// デバッグ表示開始

			printf("\n[recv]\n");								// label
			printf("time      = %lf\n", gettimeofday_sec());				// time
			printf("recv data = %s\n\n", recv_data);					// recv_data
			fflush(stdout);									// バッファフラッシュ [printf表示用]

			#endif										// デバッグ表示終了

			//-----------------------------------------------------------------------//


			memset(recv_data, 0, sizeof(recv_data));					// バッファクリア

			wait_flag = 1;									// フラグセット

		}

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


	return RTC::RTC_OK;										// 戻り値 [正常終了]

}

/*
RTC::ReturnCode_t HazardMesh::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HazardMesh::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HazardMesh::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HazardMesh::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HazardMesh::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{

  void HazardMeshInit(RTC::Manager* manager)
  {
    RTC::Properties profile(hazardmesh_spec);
    manager->registerFactory(profile,
                             RTC::Create<HazardMesh>,
                             RTC::Delete<HazardMesh>);
  }

};


