/************************************************************************************
hazardmeshirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

#ifndef HAZARDMESH_H
#define HAZARDMESH_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>


//-- [ヘッダファイル] ---------------------------------------------------------------------------//
#include "mesh.h"											// meshヘッダファイル読み込み
//-----------------------------------------------------------------------------------------------//


//-- [置換] -------------------------------------------------------------------------------------//
#define SAFETYNO	1										// セーフティカテゴリ番号
#define HAZARDNO	2										// ハザードカテゴリ番号
#define CATEGORYSIZE	18										// ハザードカテゴリサイズ
#define HAZARDCATEGORY	"1,safety,2,hazard"								// ハザードカテゴリ
#define WAITTIME	1000000										// 起動待ち時間 [us]
#define DATASIZE	256										// 最大データ文字数
#define DEBUG_CONFIGURATION										// DEBUG定義 [デバッグ用]
#define DEBUG_INPORT											// DEBUG定義 [デバッグ用]
#define DEBUG_MAPDATA											// DEBUG定義 [デバッグ用]
#define DEBUG_MESHDATA											// DEBUG定義 [デバッグ用]
#define DEBUG_RECV											// DEBUG定義 [デバッグ用]
//-----------------------------------------------------------------------------------------------//


// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "intellirobotStub.h"

// </rtc-template>

using namespace RTC;

class HazardMesh
  : public RTC::DataFlowComponentBase
{
 public:
  HazardMesh(RTC::Manager* manager);
  ~HazardMesh();

	// The initialize action (on CREATED->ALIVE transition)
	// formaer rtc_init_entry()
	virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

	// The activated action (Active state entry action)
	// former rtc_active_entry()
	virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

	// The deactivated action (Active state exit action)
	// former rtc_active_exit()
	virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

	// The execution action that is invoked periodically
	// former rtc_active_do()
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  std::string m_01_gui;
  std::string m_02_guiaddress;
  std::string m_03_ip;
  std::string m_04_sendport;
  std::string m_05_receiveport;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedMapData m_MapData;
  InPort<TimedMapData> m_MapDataIn;
  TimedMeshOrder m_MeshOrder;
  InPort<TimedMeshOrder> m_MeshOrderIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedMapOrder m_MapOrder;
  OutPort<TimedMapOrder> m_MapOrderOut;
  TimedMapData m_MeshData;
  OutPort<TimedMapData> m_MeshDataOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

	private:
		int dummy;


		//-- [変数宣言] -----------------------------------------------------------------//

		// プログラム処理
		unsigned long i;									// カウンタ格納変数
		unsigned long j;									// カウンタ格納変数

		// 入力ポート
		int trigger_flag;									// 動作トリガーフラグ格納変数
		unsigned long sec;									// タイムスタンプ格納変数 [sec]
		unsigned long nsec;									// タイムスタンプ格納変数 [nsec]
		unsigned short trigger;									// 動作トリガー格納変数
		unsigned short mode;									// アクセスモード格納変数
		char location[DATASIZE];								// 地域名格納配列
		char mapset[DATASIZE];									// 利用者マップセット名格納配列
		char type[DATASIZE];									// データタイプ格納配列
		long origin_x;										// 対象原点x座標格納変数 [mm]
		long origin_y;										// 対象原点y座標格納変数 [mm]
		unsigned long size_x;									// 対象範囲x座標格納変数 [mm]
		unsigned long size_y;									// 対象範囲y座標格納変数 [mm]
		unsigned long pixel_size_x;								// ピクセルサイズx座標格納変数 [mm]
		unsigned long pixel_size_y;								// ピクセルサイズy座標格納変数 [mm]

		unsigned long sec_2;									// タイムスタンプ格納変数 [sec]
		unsigned long nsec_2;									// タイムスタンプ格納変数 [nsec]
		unsigned short mode_2;									// アクセスモード格納変数
		char location_2[DATASIZE];								// 地域名格納配列
		char mapset_2[DATASIZE];								// 利用者マップセット名格納配列
		char type_2[DATASIZE];									// データタイプ格納配列
		long origin_x_2;									// 対象原点x座標格納変数 [mm]
		long origin_y_2;									// 対象原点y座標格納変数 [mm]
		unsigned long size_x_2;									// 対象範囲x座標格納変数 [mm]
		unsigned long size_y_2;									// 対象範囲y座標格納変数 [mm]
		char explanation_2[DATASIZE];								// 地図説明格納配列
		long map_origin_x_2;									// 地図原点x座標格納変数 [mm]
		long map_origin_y_2;									// 地図原点y座標格納変数 [mm]
		unsigned long map_size_x_2;								// 地図サイズx座標格納変数 [mm]
		unsigned long map_size_y_2;								// 地図サイズy座標格納変数 [mm]
		unsigned long pixel_size_x_2;								// ピクセルサイズx座標格納変数 [mm]
		unsigned long pixel_size_y_2;								// ピクセルサイズy座標格納変数 [mm]
		char *pcategory;									// カテゴリデータ格納配列ポインタ

		// ハザードメッシュ
		int category_num;									// カテゴリ番号格納変数
		int category_flag;									// フラグ格納変数
		unsigned long int *pmap;								// 地図データ格納配列ポインタ
		unsigned long int *phazard;								// ハザードデータ格納配列ポインタ
		unsigned long int hazard;								// ハザード番号格納変数
		unsigned long int safety;								// セーフティ番号格納変数
		unsigned long int x;									// x座標個数格納変数
		unsigned long int y;									// y座標個数格納変数
		unsigned long int hazard_x;								// ハザードx座標個数格納変数
		unsigned long int hazard_y;								// ハザードy座標個数格納変数
		char category_buf[2][DATASIZE];								// カテゴリデータ一時保存配列

		// GUIプログラム
		int gui_flag;										// GUI動作フラグ格納変数
		char gui[4];										// GUI動作格納配列
		char gui_addr[256];									// GUIプログラムアドレス格納配列
		char ip[16];										// IPアドレス格納配列
		char send_port[6];									// 送信ポート番号格納配列
		char recv_port[6];									// 受信ポート番号格納配列
		char command[256];									// コマンドライン引数格納配列

		// UDP通信
		char *psend;										// 送信データ格納ポインタ
		char send_data[3 * BUFSIZE];								// 送信データ格納配列
		char recv_data[BUFSIZE];								// 受信データ格納配列
		int send_sock;										// ソケットディスクリプタ格納変数
		int recv_sock;										// ソケットディスクリプタ格納変数
		int recv_len;										// 受信サイズ格納変数
		struct sockaddr_in send_addr;								// アドレス格納構造体
		struct sockaddr_in recv_addr;								// アドレス格納構造体
		struct sigaction saio;									// シグナル受信時動作設定格納構造体
		socklen_t addr_size;									// アドレス格納構造体サイズ格納変数

		//-------------------------------------------------------------------------------//


};


extern "C"
{
  void HazardMeshInit(RTC::Manager* manager);
};

#endif // HAZARDMESH_H
